@@ -191,7 +191,8 @@ windows, it is abysmally slow, do not expect more than 5..20 forks/s on
 that sucky platform (note this uses perl's pseudo-threads, so avoid those
 like the plague).
 
-Example: poor man's async disk I/O (better use L<IO::AIO>).
+Example: poor man's async disk I/O (better use L<AnyEvent::IO> together
+with L<IO::AIO>).
 
    fork_call {
       open my $fh, "</etc/passwd"
@@ -605,6 +606,7 @@ sub run_cmd {
             $cv->begin;
 
             fcntl $pr, AnyEvent::F_SETFD, AnyEvent::FD_CLOEXEC;
+            fh_nonblocking $pr, 1;
             my $w; $w = AE::io $pr, 0,
                "SCALAR" eq ref $ob
                   ? sub {
@@ -649,6 +651,7 @@ sub run_cmd {
             }
 
             fcntl $pw, AnyEvent::F_SETFD, AnyEvent::FD_CLOEXEC;
+            fh_nonblocking $pw, 1;
             my $w; $w = AE::io $pw, 1, sub {
                my $len = syswrite $pw, $data;
 
@@ -799,7 +802,7 @@ sub idn_nameprep($;$) {
       # load the mapping data
       unless (defined $uts46_imap) {
          require Unicode::Normalize;
-         require "lib/AnyEvent/Util/uts46data.pl";
+         require "AnyEvent/Util/uts46data.pl";
       }
 
       # uts46 nameprep
@@ -866,7 +869,7 @@ sub idn_nameprep($;$) {
    if (/[^0-9a-z\-.]/) {
       # load the mapping data
       unless (defined $uts46_imap) {
-         require "lib/AnyEvent/Util/uts46data.pl";
+         require "AnyEvent/Util/uts46data.pl";
       }
 
       vec $uts46_valid, ord, 1
